/* -*- Mode: C; indent-tabs-mode:nil; c-basic-offset: 8-*- */

/*
 *This file is part of MlView.
 *
 *MlView is free software; you can redistribute 
 *it and/or modify it under the terms of 
 *the GNU General Public License as published by the 
 *Free Software Foundation; either version 2, 
 *or (at your option) any later version.
 *
 *GNU MlView is distributed in the hope that it will 
 *be useful, but WITHOUT ANY WARRANTY; 
 *without even the implied warranty of 
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *See the GNU General Public License for more details.
 *
 *You should have received a copy of the 
 *GNU General Public License along with MlView; 
 *see the file COPYING. 
 *If not, write to the Free Software Foundation, 
 *Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *See COPYRIGHT file for copyright information.
 */

#include "mlview-utils.h"
#include "mlview-completion-helpers.h"

/**
 *@file
 *Some helpers to build #MlViewICompletionWidget implementations
 *based on four GtkTreeView.
 */

static void
mlview_completion_views_update_list (GtkListStore *a_store, GList *a_list)
{
        GtkTreeIter iter;

        gtk_list_store_clear (a_store);
        while (a_list) {
                gtk_list_store_append (a_store, &iter);
                gtk_list_store_set (a_store, &iter, 0, a_list->data, -1);
                a_list = g_list_next (a_list);
        }
}

/**
 *Updates #MlViewCompletionModels possible
 *attributes list.
 *@param a_models the #MlViewCompletionModels instance.
 *@param a_list the possible attributes list (gchar *).
 */
void
mlview_completion_views_update_attributes_list (MlViewCompletionModels *a_models, GList *a_list)
{
        GtkListStore *store;

        g_return_if_fail (a_models);
        store = GTK_LIST_STORE (a_models->feasible_attributes);
        g_return_if_fail (store);
        mlview_completion_views_update_list (store, a_list);
}

/**
 *Updates #MlViewCompletionModels possible
 *previous siblings list.
 *@param a_models the #MlViewCompletionModels instance.
 *@param a_list the possible previous siblings list (gchar *).
 */
void
mlview_completion_views_update_prev_siblings_list (MlViewCompletionModels *a_models, GList *a_list)
{
        GtkListStore *store;

        g_return_if_fail (a_models);
        store = GTK_LIST_STORE (a_models->feasible_prev_siblings);
        g_return_if_fail (store);
        mlview_completion_views_update_list (store, a_list);
}

/**
 *Updates #MlViewCompletionModels possible
 *next siblings list.
 *@param a_models the #MlViewCompletionModels instance.
 *@param a_list the possible next siblings list (gchar *).
 */
void
mlview_completion_views_update_next_siblings_list (MlViewCompletionModels *a_models, GList *a_list)
{
        GtkListStore *store;

        g_return_if_fail (a_models);
        store = GTK_LIST_STORE (a_models->feasible_next_siblings);
        g_return_if_fail (store);
        mlview_completion_views_update_list (store, a_list);
}

/**
 *Updates #MlViewCompletionModels possible
 *children list.
 *@param a_models the #MlViewCompletionModels instance.
 *@param a_list the possible children list (gchar *).
 */
void
mlview_completion_views_update_children_list (MlViewCompletionModels *a_models, GList *a_list)
{
        GtkListStore *store;

        g_return_if_fail (a_models);
        store = GTK_LIST_STORE (a_models->feasible_children);
        g_return_if_fail (store);
        mlview_completion_views_update_list (store, a_list);
}

static void        
feasible_child_selected_cb (GtkTreeSelection *a_tree_selection,
                            gpointer a_user_data)
{
        MlViewCompletionModels *models;
        GtkTreeIter iter;
        gchar *str = NULL;

        g_return_if_fail (a_tree_selection && a_user_data);
        models = (MlViewCompletionModels*)a_user_data;
        g_return_if_fail (models->ptr && G_IS_OBJECT (models->ptr));
        if (gtk_tree_selection_get_selected (a_tree_selection, NULL, &iter)) {
                gtk_tree_selection_unselect_all (models->feasible_prev_siblings_selection);
                gtk_tree_selection_unselect_all (models->feasible_next_siblings_selection);
                gtk_tree_selection_unselect_all (models->feasible_attributes_selection);
                gtk_tree_model_get (models->feasible_children, &iter, 0, &str, -1);
                g_signal_emit_by_name (models->ptr, "selected-a-possible-child", 
                                       str);
        }
}

static void
feasible_prev_sibling_selected_cb (GtkTreeSelection *a_tree_selection,
                                   gpointer a_user_data)
{
        MlViewCompletionModels *models;
        GtkTreeIter iter;
        gchar *str = NULL;
        
        g_return_if_fail (a_tree_selection && a_user_data);
        models = (MlViewCompletionModels*)a_user_data;
        g_return_if_fail (models->ptr && G_IS_OBJECT (models->ptr));
        if (gtk_tree_selection_get_selected (a_tree_selection, NULL, &iter)) {
                gtk_tree_selection_unselect_all (models->feasible_children_selection);
                gtk_tree_selection_unselect_all (models->feasible_next_siblings_selection);
                gtk_tree_selection_unselect_all (models->feasible_attributes_selection);
                gtk_tree_model_get (models->feasible_prev_siblings, &iter, 0, &str, -1);
                g_signal_emit_by_name (models->ptr, "selected-a-possible-prev-sibling", 
                                       str);
        }
}

static void
feasible_next_sibling_selected_cb (GtkTreeSelection *a_tree_selection,
                                   gpointer a_user_data)
{
        MlViewCompletionModels *models;
        GtkTreeIter iter;
        gchar *str = NULL;
        
        g_return_if_fail (a_tree_selection && a_user_data);
        models = (MlViewCompletionModels*)a_user_data;
        g_return_if_fail (models->ptr && G_IS_OBJECT (models->ptr));
        if (gtk_tree_selection_get_selected (a_tree_selection, NULL, &iter)) {
                gtk_tree_selection_unselect_all (models->feasible_prev_siblings_selection);
                gtk_tree_selection_unselect_all (models->feasible_children_selection);
                gtk_tree_selection_unselect_all (models->feasible_attributes_selection);
                gtk_tree_model_get (models->feasible_next_siblings, &iter, 0, &str, -1);
                g_signal_emit_by_name (models->ptr, "selected-a-possible-next-sibling", 
                                       str);
        }
}

static void
feasible_attribute_selected_cb (GtkTreeSelection *a_tree_selection,
                                gpointer a_user_data)
{
        MlViewCompletionModels *models;
        GtkTreeIter iter;
        gchar *str = NULL;
        
        g_return_if_fail (a_tree_selection && a_user_data);
        models = (MlViewCompletionModels*)a_user_data;
        g_return_if_fail (models->ptr && G_IS_OBJECT (models->ptr));
        if (gtk_tree_selection_get_selected (a_tree_selection, NULL, &iter)) {
                gtk_tree_selection_unselect_all (models->feasible_prev_siblings_selection);
                gtk_tree_selection_unselect_all (models->feasible_next_siblings_selection);
                gtk_tree_selection_unselect_all (models->feasible_children_selection);
                gtk_tree_model_get (models->feasible_attributes, &iter, 0, &str, -1);
                g_signal_emit_by_name (models->ptr, "selected-a-possible-attribute", 
                                       str);
        }
}

/**
 *Creates #MlViewCompletionViews and #MlViewCompletionModels
 *associated with an #MlViewICompletionWidget, and connects the 
 *signals.
 *@param a_ptr the current #MlViewICompletionWidget instance.
 *@param a_models an empty #MlViewCompletionModels.
 *@param a_views an empty #MlViewCompletionViews.
 */
void
mlview_completion_views_create (GObject *a_ptr, 
                                MlViewCompletionModels *a_models, 
				MlViewCompletionViews *a_views)
{
        GtkCellRenderer *renderer;
        GtkTreeSelection *selection;
        
        g_return_if_fail (a_ptr && G_IS_OBJECT (a_ptr) && a_models && a_views);
        
        a_models->ptr = a_ptr;
        
        a_models->feasible_children = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
        a_views->feasible_children = gtk_tree_view_new_with_model (a_models->feasible_children);
        renderer = gtk_cell_renderer_text_new ();
        gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (a_views->feasible_children), 1,
                                                     _("Possible children"), renderer, "text", 0, NULL);
        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (a_views->feasible_children));
        g_signal_connect (G_OBJECT (selection), "changed", 
                          G_CALLBACK (feasible_child_selected_cb),
                          a_models);
        a_models->feasible_children_selection = selection;
        
        a_models->feasible_prev_siblings = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
        a_views->feasible_prev_siblings = gtk_tree_view_new_with_model (a_models->feasible_prev_siblings);
        renderer = gtk_cell_renderer_text_new ();
        gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (a_views->feasible_prev_siblings), 1,
                                                     _("Possible previous siblings"), renderer, "text", 0, 
                                                     NULL);
        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (a_views->feasible_prev_siblings));
        g_signal_connect (G_OBJECT (selection), "changed",
                          G_CALLBACK (feasible_prev_sibling_selected_cb),
                          a_models);
        a_models->feasible_prev_siblings_selection = selection;
        
        a_models->feasible_next_siblings = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
        a_views->feasible_next_siblings = gtk_tree_view_new_with_model (a_models->feasible_next_siblings);
        renderer = gtk_cell_renderer_text_new ();
        gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (a_views->feasible_next_siblings), 1,
                                                     _("Possible next siblings"), renderer, 
                                                     "text", 0, NULL);
        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (a_views->feasible_next_siblings));
        g_signal_connect (G_OBJECT (selection), "changed",
                          G_CALLBACK (feasible_next_sibling_selected_cb),
                          a_models);
        a_models->feasible_next_siblings_selection = selection;
        
        a_models->feasible_attributes = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
        a_views->feasible_attributes = gtk_tree_view_new_with_model (a_models->feasible_attributes);
        renderer = gtk_cell_renderer_text_new ();
        gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (a_views->feasible_attributes), 1,
                                                     _("Possible attributes"), renderer, "text", 0, NULL);
        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (a_views->feasible_attributes));
        g_signal_connect (G_OBJECT (selection), "changed",
                          G_CALLBACK (feasible_attribute_selected_cb),
                          a_models);
        a_models->feasible_attributes_selection = selection;
}
