/* -*- Mode: C; indent-tabs-mode:nil; c-basic-offset: 8-*- */

/*
 *This file is part of MlView.
 *
 *MlView is free software; you can redistribute 
 *it and/or modify it under the terms of 
 *the GNU General Public License as published by the 
 *Free Software Foundation; either version 2, 
 *or (at your option) any later version.
 *
 *GNU MlView is distributed in the hope that it will 
 *be useful, but WITHOUT ANY WARRANTY; 
 *without even the implied warranty of 
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *See the GNU General Public License for more details.
 *
 *You should have received a copy of the 
 *GNU General Public License along with MlView; 
 *see the file COPYING. 
 *If not, write to the Free Software Foundation, 
 *Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *See COPYRIGHT file for copyright information.
 */

#include "mlview-icompletion-widget.h"
#include "mlview-completion-helpers.h"
#include "mlview-completion-notebook.h"
#include "mlview-utils.h"

/**
 *@file
 *The definition of the #MlViewCompletionNotebook class.
 */

#define PRIVATE(completion_notebook) ((completion_notebook)->priv)

struct
_MlViewCompletionNotebookPrivate {
        MlViewCompletionModels *models;
        
        gboolean dispose_has_run;
};

static GtkNotebook *gv_parent_class;

static void
mlview_completion_notebook_finalize (GObject *a_this)
{
        MlViewCompletionNotebook *notebook;
        
        g_return_if_fail (a_this && MLVIEW_COMPLETION_NOTEBOOK (a_this));
        notebook = MLVIEW_COMPLETION_NOTEBOOK (a_this);
        g_return_if_fail (PRIVATE (notebook));
        g_free (PRIVATE (notebook));
        PRIVATE (notebook) = NULL;
        
        if (gv_parent_class 
            && G_OBJECT_CLASS (gv_parent_class)->finalize) {
                G_OBJECT_CLASS (gv_parent_class)->finalize (a_this) ;
        }
}

static void
mlview_completion_notebook_dispose (GObject *a_this)
{
        MlViewCompletionNotebook *notebook;
        
        g_return_if_fail (a_this && MLVIEW_COMPLETION_NOTEBOOK (a_this));
        notebook = MLVIEW_COMPLETION_NOTEBOOK (a_this);
        g_return_if_fail (PRIVATE (notebook));
        
        if (PRIVATE (notebook)->models) {
                g_free (PRIVATE (notebook)->models);
                PRIVATE (notebook)->models = NULL;
        }
        
        if (PRIVATE (notebook)->dispose_has_run == TRUE) 
                return;
        
        PRIVATE (notebook)->dispose_has_run = TRUE;
        
        if (gv_parent_class 
            && G_OBJECT_CLASS (gv_parent_class)->dispose)
                G_OBJECT_CLASS (gv_parent_class)->dispose (a_this);
}

static void
mlview_completion_notebook_class_init (MlViewCompletionNotebookClass *a_klass)
{
        GObjectClass *gobject_class = NULL;
        
        gobject_class = G_OBJECT_CLASS (a_klass);
        g_return_if_fail (gobject_class);
        
        gobject_class->dispose = mlview_completion_notebook_dispose;
        gobject_class->finalize = mlview_completion_notebook_finalize;
        
        gv_parent_class =
                gtk_type_class (GTK_TYPE_NOTEBOOK);
}

static void
mlview_completion_notebook_init (MlViewCompletionNotebook *a_this)
{
        GtkWidget *scrolled;
        MlViewCompletionViews views;
        
        if (PRIVATE (a_this) == NULL) {
                PRIVATE (a_this) = g_malloc0 (sizeof (MlViewCompletionNotebookPrivate));
                PRIVATE (a_this)->models = g_malloc0 (sizeof (MlViewCompletionModels));
        }
        
        gtk_notebook_set_tab_pos (GTK_NOTEBOOK (a_this), GTK_POS_BOTTOM);

        mlview_completion_views_create (G_OBJECT (a_this), PRIVATE (a_this)->models, &views);
        
        gtk_notebook_append_page (GTK_NOTEBOOK (a_this), views.feasible_children, 
                                  gtk_label_new (_("Possible children")));
        gtk_notebook_append_page (GTK_NOTEBOOK (a_this), views.feasible_prev_siblings,
                                  gtk_label_new (_("Possible previous siblings")));
        gtk_notebook_append_page (GTK_NOTEBOOK (a_this), views.feasible_next_siblings,
                                  gtk_label_new (_("Possible next siblings")));
        gtk_notebook_append_page (GTK_NOTEBOOK (a_this), views.feasible_attributes,
                                  gtk_label_new (_("Possible attributes")));
}

static void
mlview_completion_notebook_update_feasible_attributes_list (MlViewICompletionWidget *a_iface,
                                                         GList *a_list)
{
        MlViewCompletionNotebook *notebook;

        g_return_if_fail (a_iface);
        notebook = MLVIEW_COMPLETION_NOTEBOOK (a_iface);
        g_return_if_fail (notebook && PRIVATE (notebook));

        mlview_completion_views_update_attributes_list (PRIVATE (notebook)->models,
                                                        a_list);
}

static void
mlview_completion_notebook_update_feasible_prev_siblings_list (MlViewICompletionWidget *a_iface,
                                                            GList *a_list)
{
        MlViewCompletionNotebook *notebook;

        g_return_if_fail (a_iface);
        notebook = MLVIEW_COMPLETION_NOTEBOOK (a_iface);
        g_return_if_fail (notebook && PRIVATE (notebook));

        mlview_completion_views_update_prev_siblings_list 
                (PRIVATE (notebook)->models,
                 a_list);
}

static void
mlview_completion_notebook_update_feasible_next_siblings_list (MlViewICompletionWidget *a_iface,
                                                            GList *a_list)
{
        MlViewCompletionNotebook *notebook;

        g_return_if_fail (a_iface);
        notebook = MLVIEW_COMPLETION_NOTEBOOK (a_iface);
        g_return_if_fail (notebook && PRIVATE (notebook));

        mlview_completion_views_update_next_siblings_list 
                (PRIVATE (notebook)->models,
                 a_list);
}

static void
mlview_completion_notebook_update_feasible_children_list (MlViewICompletionWidget *a_iface,
                                                       GList *a_list)
{
        MlViewCompletionNotebook *notebook;

        g_return_if_fail (a_iface);
        notebook = MLVIEW_COMPLETION_NOTEBOOK (a_iface);
        g_return_if_fail (notebook && PRIVATE (notebook));

        mlview_completion_views_update_children_list 
                (PRIVATE (notebook)->models,
                 a_list);
}

static void
mlview_completion_notebook_icompletion_widget_init (MlViewICompletionWidget *a_iface)
{
        g_return_if_fail (a_iface);
        
        a_iface->update_feasible_attributes_list = mlview_completion_notebook_update_feasible_attributes_list;
        a_iface->update_feasible_prev_siblings_list = 
                mlview_completion_notebook_update_feasible_prev_siblings_list;
        a_iface->update_feasible_next_siblings_list = 
                mlview_completion_notebook_update_feasible_next_siblings_list;
        a_iface->update_feasible_children_list = mlview_completion_notebook_update_feasible_children_list;
}

GType
mlview_completion_notebook_get_type (void)
{
        static GType mlview_completion_notebook_type = 0;
        
        if (!mlview_completion_notebook_type) {
                static const GTypeInfo
                        mlview_completion_notebook_type_info = {
                                sizeof (MlViewCompletionNotebookClass),
                                NULL,   /* base_init */
                                NULL,   /* base_finalize */
                                (GClassInitFunc)
                                mlview_completion_notebook_class_init,
                                NULL,   /* class_finalize */
                                NULL,   /* class_data */
                                sizeof (MlViewCompletionNotebook),
                                0,
                                (GInstanceInitFunc)
                                mlview_completion_notebook_init
                        };
                static const GInterfaceInfo
                        mlview_icompletion_widget_iface_info = {
                                (GInterfaceInitFunc) mlview_completion_notebook_icompletion_widget_init,
                                NULL,
                                NULL
                        };
                mlview_completion_notebook_type =
                        g_type_register_static (GTK_TYPE_NOTEBOOK,
                                                "MlViewCompletionNotebook",
                                                &mlview_completion_notebook_type_info,
                                                0);
                g_type_add_interface_static (mlview_completion_notebook_type,
                                             MLVIEW_TYPE_ICOMPLETION_WIDGET,
                                             &mlview_icompletion_widget_iface_info);
        }
        return mlview_completion_notebook_type;
}
