/* -*- Mode: C; indent-tabs-mode:nil; c-basic-offset: 8-*- */

/*
 *This file is part of MlView.
 *
 *MlView is free software; you can redistribute 
 *it and/or modify it under the terms of 
 *the GNU General Public License as published by the 
 *Free Software Foundation; either version 2, 
 *or (at your option) any later version.
 *
 *GNU MlView is distributed in the hope that it will 
 *be useful, but WITHOUT ANY WARRANTY; 
 *without even the implied warranty of 
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *See the GNU General Public License for more details.
 *
 *You should have received a copy of the 
 *GNU General Public License along with MlView; 
 *see the file COPYING. 
 *If not, write to the Free Software Foundation, 
 *Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *See COPYRIGHT file for copyright information.
 */

#include "mlview-icompletion-widget.h"
#include "mlview-completion-helpers.h"
#include "mlview-completion-table.h"

/**
 *@file
 *The definition of the #MlViewCompletionTable class.
 */

#define PRIVATE(completion_table) ((completion_table)->priv)

struct
_MlViewCompletionTablePrivate {
        MlViewCompletionModels *models;
        
        gboolean dispose_has_run;
};

static GtkTable *gv_parent_class;

static void
mlview_completion_table_finalize (GObject *a_this)
{
        MlViewCompletionTable *table;
        
        g_return_if_fail (a_this && MLVIEW_COMPLETION_TABLE (a_this));
        table = MLVIEW_COMPLETION_TABLE (a_this);
        g_return_if_fail (PRIVATE (table));
        g_free (PRIVATE (table));
        PRIVATE (table) = NULL;
        
        if (gv_parent_class 
            && G_OBJECT_CLASS (gv_parent_class)->finalize) {
                G_OBJECT_CLASS (gv_parent_class)->finalize (a_this) ;
        }
}

static void
mlview_completion_table_dispose (GObject *a_this)
{
        MlViewCompletionTable *table;
        
        g_return_if_fail (a_this && MLVIEW_COMPLETION_TABLE (a_this));
        table = MLVIEW_COMPLETION_TABLE (a_this);
        g_return_if_fail (PRIVATE (table));
        
        if (PRIVATE (table)->models) {
                g_free (PRIVATE (table)->models);
                PRIVATE (table)->models = NULL;
        }
        
        if (PRIVATE (table)->dispose_has_run == TRUE) 
                return;
        
        PRIVATE (table)->dispose_has_run = TRUE;
        
        if (gv_parent_class 
            && G_OBJECT_CLASS (gv_parent_class)->dispose)
                G_OBJECT_CLASS (gv_parent_class)->dispose (a_this);
}

static void
mlview_completion_table_class_init (MlViewCompletionTableClass *a_klass)
{
        GObjectClass *gobject_class = NULL;
        
        gobject_class = G_OBJECT_CLASS (a_klass);
        g_return_if_fail (gobject_class);
        
        gobject_class->dispose = mlview_completion_table_dispose;
        gobject_class->finalize = mlview_completion_table_finalize;
        
        gv_parent_class =
                gtk_type_class (GTK_TYPE_TABLE);
}

static void
mlview_completion_table_init (MlViewCompletionTable *a_this)
{
        GtkWidget *scrolled;
        MlViewCompletionViews views;
        
        if (PRIVATE (a_this) == NULL) {
                PRIVATE (a_this) = g_malloc0 (sizeof (MlViewCompletionTablePrivate));
                PRIVATE (a_this)->models = g_malloc0 (sizeof (MlViewCompletionModels));
        }
        
        mlview_completion_views_create (G_OBJECT (a_this), PRIVATE (a_this)->models, &views);
        
        gtk_table_resize (GTK_TABLE (a_this), 2, 2);
        scrolled = gtk_scrolled_window_new (NULL, NULL);
        gtk_container_add (GTK_CONTAINER (scrolled), views.feasible_children);
        gtk_table_attach_defaults (GTK_TABLE (a_this), scrolled, 0, 1, 0, 1);
        
        scrolled = gtk_scrolled_window_new (NULL, NULL);
        gtk_container_add (GTK_CONTAINER (scrolled), views.feasible_prev_siblings);
        gtk_table_attach_defaults (GTK_TABLE (a_this), scrolled, 1, 2, 0, 1);
        
        scrolled = gtk_scrolled_window_new (NULL, NULL);
        gtk_container_add (GTK_CONTAINER (scrolled), views.feasible_next_siblings);
        gtk_table_attach_defaults (GTK_TABLE (a_this), scrolled, 1, 2, 1, 2);
        
        scrolled = gtk_scrolled_window_new (NULL, NULL);
        gtk_container_add (GTK_CONTAINER (scrolled), views.feasible_attributes);
        gtk_table_attach_defaults (GTK_TABLE (a_this), scrolled, 0, 1, 1, 2);
}

static void
mlview_completion_table_update_feasible_attributes_list (MlViewICompletionWidget *a_iface,
                                                         GList *a_list)
{
        MlViewCompletionTable *table;

        g_return_if_fail (a_iface);
        table = MLVIEW_COMPLETION_TABLE (a_iface);
        g_return_if_fail (table && PRIVATE (table));

        mlview_completion_views_update_attributes_list (PRIVATE (table)->models,
                                                        a_list);
}

static void
mlview_completion_table_update_feasible_prev_siblings_list (MlViewICompletionWidget *a_iface,
                                                            GList *a_list)
{
        MlViewCompletionTable *table;

        g_return_if_fail (a_iface);
        table = MLVIEW_COMPLETION_TABLE (a_iface);
        g_return_if_fail (table && PRIVATE (table));

        mlview_completion_views_update_prev_siblings_list 
                (PRIVATE (table)->models,
                 a_list);
}

static void
mlview_completion_table_update_feasible_next_siblings_list (MlViewICompletionWidget *a_iface,
                                                            GList *a_list)
{
        MlViewCompletionTable *table;

        g_return_if_fail (a_iface);
        table = MLVIEW_COMPLETION_TABLE (a_iface);
        g_return_if_fail (table && PRIVATE (table));

        mlview_completion_views_update_next_siblings_list 
                (PRIVATE (table)->models,
                 a_list);
}

static void
mlview_completion_table_update_feasible_children_list (MlViewICompletionWidget *a_iface,
                                                       GList *a_list)
{
        MlViewCompletionTable *table;

        g_return_if_fail (a_iface);
        table = MLVIEW_COMPLETION_TABLE (a_iface);
        g_return_if_fail (table && PRIVATE (table));

        mlview_completion_views_update_children_list 
                (PRIVATE (table)->models,
                 a_list);
}

static void
mlview_completion_table_icompletion_widget_init (MlViewICompletionWidget *a_iface)
{
        g_return_if_fail (a_iface);
        
        a_iface->update_feasible_attributes_list = mlview_completion_table_update_feasible_attributes_list;
        a_iface->update_feasible_prev_siblings_list = 
                mlview_completion_table_update_feasible_prev_siblings_list;
        a_iface->update_feasible_next_siblings_list = 
                mlview_completion_table_update_feasible_next_siblings_list;
        a_iface->update_feasible_children_list = mlview_completion_table_update_feasible_children_list;
}

GType
mlview_completion_table_get_type (void)
{
        static GType mlview_completion_table_type = 0;
        
        if (!mlview_completion_table_type) {
                static const GTypeInfo
                        mlview_completion_table_type_info = {
                                sizeof (MlViewCompletionTableClass),
                                NULL,   /* base_init */
                                NULL,   /* base_finalize */
                                (GClassInitFunc)
                                mlview_completion_table_class_init,
                                NULL,   /* class_finalize */
                                NULL,   /* class_data */
                                sizeof (MlViewCompletionTable),
                                0,
                                (GInstanceInitFunc)
                                mlview_completion_table_init
                        };
                static const GInterfaceInfo
                        mlview_icompletion_widget_iface_info = {
                                (GInterfaceInitFunc) mlview_completion_table_icompletion_widget_init,
                                NULL,
                                NULL
                        };
                mlview_completion_table_type =
                        g_type_register_static (GTK_TYPE_TABLE,
                                                "MlViewCompletionTable",
                                                &mlview_completion_table_type_info,
                                                0);
                g_type_add_interface_static (mlview_completion_table_type,
                                             MLVIEW_TYPE_ICOMPLETION_WIDGET,
                                             &mlview_icompletion_widget_iface_info);
        }
        return mlview_completion_table_type;
}
